package mydb

import "git.hexq.cn/tiglog/mydb/internal/adapter"

// AndExpr represents an expression joined by a logical conjuction (AND).
type AndExpr struct {
	*adapter.LogicalExprGroup
}

// And adds more expressions to the group.
func (a *AndExpr) And(andConds ...LogicalExpr) *AndExpr {
	var fn func(*[]LogicalExpr) error
	if len(andConds) > 0 {
		fn = func(in *[]LogicalExpr) error {
			*in = append(*in, andConds...)
			return nil
		}
	}
	return &AndExpr{a.LogicalExprGroup.Frame(fn)}
}

// Empty returns false if the expressions has zero conditions.
func (a *AndExpr) Empty() bool {
	return a.LogicalExprGroup.Empty()
}

// And joins conditions under logical conjunction. Conditions can be
// represented by `db.Cond{}`, `db.Or()` or `db.And()`.
//
// Examples:
//
//	// name = "Peter" AND last_name = "Parker"
//	db.And(
//		db.Cond{"name": "Peter"},
//		db.Cond{"last_name": "Parker "},
//	)
//
//	// (name = "Peter" OR name = "Mickey") AND last_name = "Mouse"
//	db.And(
//		db.Or(
//			db.Cond{"name": "Peter"},
//			db.Cond{"name": "Mickey"},
//		),
//		db.Cond{"last_name": "Mouse"},
//	)
func And(conds ...LogicalExpr) *AndExpr {
	return &AndExpr{adapter.NewLogicalExprGroup(adapter.LogicalOperatorAnd, conds...)}
}

var _ = adapter.LogicalExpr(&AndExpr{})
